;;; oni-php.el --- PHP Configuration                 -*- lexical-binding: t; -*-

;; Copyright (C) 2019  Tom Willemse

;; Author: Tom Willemse <tom@ryuslash.org>
;; Keywords: local
;; Version: 20190218221251
;; Package-Requires: (php-mode oni-flycheck oni-company ggtags fill-column-indicator fic-mode hydra company-php)

;; This program is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.

;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <https://www.gnu.org/licenses/>.

;;; Commentary:

;; My PHP mode configuration. Includes rules for aligning variables according to
;; the WordPress coding standards.

;;; Code:

(require 'align)
(require 'company)
(require 'php-mode)
(require 'whitespace)

(defun oni-php--set-require-final-newline ()
  "Set `require-final-newline' to t.
This is necessary because the PHP mode configuration sets this to
nil for some reason."
  (setq require-final-newline t))

(defun oni-php--whitespace-mode ()
  "Enable whitespace mode with only tabs showing."
  (setq-local whitespace-style '(face tabs))
  (whitespace-mode))

(defhydra php-mode-hydra (:color blue)
  ("a" align-current "Align current selection"))

(add-hook 'php-mode-hook #'oni-php--set-require-final-newline)
(add-hook 'php-mode-hook #'oni-php--whitespace-mode)
(add-hook 'php-mode-hook 'company-mode)
(add-hook 'php-mode-hook 'electric-indent-local-mode)
(add-hook 'php-mode-hook 'electric-pair-local-mode)
(add-hook 'php-mode-hook 'fci-mode)
(add-hook 'php-mode-hook 'fic-mode)
(add-hook 'php-mode-hook 'flycheck-mode)
(add-hook 'php-mode-hook 'ggtags-mode)

(with-eval-after-load 'company
  (add-to-list 'company-backends 'company-ac-php-backend))

(define-key php-mode-map (kbd "C-c m") #'php-mode-hydra/body)

;; In PHP code it's nice to have any ~=>~ aligned.

;; <?php
;; array(
;;     'foo'  => 'bar',
;;     'frob' => 'baz'
;; );
;; ?>

(add-to-list 'align-rules-list
             `(php-array-arrow
               (regexp . ,(rx any (group (zero-or-more whitespace)) "=>" any))
               (group . (1))
               (modes . '(php-mode web-mode))
               (repeat . t)))

;; The WordPress coding standards specify that multiple assignments
;; should have their assignment operators aligned.

;; <?php
;; $variable          = "value";
;; $other_variable    = "other value";
;; $one_more_variable = "one more variable";

(add-to-list 'align-rules-list
             `(php-assignment-equals
               (regexp . ,(rx any (group (zero-or-more whitespace)) "="
                              (zero-or-more whitespace) any))
               (group . (1))
               (modes . '(php-mode web-mode))
               (repeat . t)))

;;;###autoload(with-eval-after-load 'php-mode (require 'oni-php))
;;;###autoload(add-to-list 'auto-mode-alist '("\\.inc\\'" . php-mode))
;;;###autoload(add-to-list 'auto-mode-alist '("\\.module\\'" . php-mode))
;;;###autoload(with-eval-after-load 'grep (add-to-list 'grep-files-aliases '("php" . "*.php *.inc *.module")))

(provide 'oni-php)
;;; oni-php.el ends here
